use anyhow::{bail, Result};
use serde::{Deserialize, Serialize};
use serde_json::json;
use std::{fs, path};

#[derive(Deserialize)]
pub struct ConfigFile {
    // Hostname to connect with the api
    pub hostname: String,
    // Token used in the api to authenticate
    pub token: String,
    // Path to where to store some generated configuration values
    // defaults to ./data.toml
    pub config_path: Option<String>,
    // Data Path
    // Path to where the data is mounted
    pub data_path: String,
}

#[derive(Deserialize, Serialize)]
pub struct RunnerData {
    pub id: String,
}

pub async fn load_runner_data(config: &ConfigFile) -> Result<RunnerData> {
    let data_path = config.config_path.as_ref().unwrap();
    let data_path = path::Path::new(&*data_path);

    if data_path.exists() {
        let runner_data = fs::read_to_string(data_path)?;
        Ok(toml::from_str(&runner_data)?)
    } else {
        let client = reqwest::Client::new();
        let to_send = json!({
            "token": config.token,
              "type": 1,
        });

        let register_resp = client
            .post(format!("{}/tasks/runner/register", config.hostname))
            .header("token", &config.token)
            .body(to_send.to_string())
            .send()
            .await?;

        if register_resp.status() != 200 {
            bail!(format!("Could not create runner {:#?}", register_resp));
        }

        let runner_data: RunnerData = register_resp.json().await?;

        fs::write(data_path, toml::to_string(&runner_data)?)
            .expect("Faield to save data for runner");

        Ok(runner_data)
    }
}
