use std::sync::Arc;

use anyhow::{bail, Result};
use serde::Deserialize;
use serde_json::json;
use serde_repr::Deserialize_repr;

use crate::{ConfigFile, RunnerData};

#[derive(Clone, Copy, Deserialize_repr, Debug)]
#[repr(i8)]
pub enum TaskStatus {
    FailedRunning = -2,
    FailedCreation = -1,
    Preparing = 0,
    Todo = 1,
    PickedUp = 2,
    Running = 3,
    Done = 4,
}

#[derive(Clone, Copy, Deserialize_repr, Debug)]
#[repr(i8)]
pub enum TaskType {
    Classification = 1,
    Training = 2,
    Retraining = 3,
    DeleteUser = 4,
}

#[derive(Deserialize, Debug)]
pub struct Task {
    pub id: String,
    pub user_id: String,
    pub model_id: String,
    pub status: TaskStatus,
    pub status_message: String,
    pub user_confirmed: i8,
    pub compacted: i8,
    #[serde(alias = "type")]
    pub task_type: TaskType,
    pub extra_task_info: String,
    pub result: String,
    pub created: String,
}

pub async fn fail_task(
    task: &Task,
    config: Arc<ConfigFile>,
    runner_data: Arc<RunnerData>,
    reason: &str,
) -> Result<()> {
    println!("Marking Task as faield");

    let client = reqwest::Client::new();

    let to_send = json!({
        "id": runner_data.id,
         "taskId": task.id,
          "reason": reason
    });

    let resp = client
        .post(format!("{}/tasks/runner/fail", config.hostname))
        .header("token", &config.token)
        .body(to_send.to_string())
        .send()
        .await?;

    if resp.status() != 200 {
        println!("Failed to update status of task");
        bail!("Failed to update status of task");
    }

    Ok(())
}

impl Task {
    pub async fn fail(
        self: &mut Task,
        config: Arc<ConfigFile>,
        runner_data: Arc<RunnerData>,
        reason: &str,
    ) -> Result<()> {
        fail_task(self, config, runner_data, reason).await?;
        self.status = TaskStatus::FailedRunning;
        self.status_message = reason.to_string();
        Ok(())
    }
}
